(* gram.ml4 *)

open Syntax

type tags =
  | Const of string
  | TSize of int
  | TColor of color

let give_tag doc = function
  | Const "B" -> B doc
  | Const "EM" -> EM doc
  | Const "I" -> I doc
  | Const "PL" -> PL doc
  | Const "St" -> St doc
  | Const "TT" -> TT doc
  | Const "U" -> U doc
  | TSize i -> Size (i,doc)
  | TColor c -> Color (c,doc)
  | _ -> failwith "Unknown tag"

let rec make_seq = function
  | [] -> Nil
  | a::tl ->
    if tl = [] then a
    else Seq (a,make_seq tl)

let gram = Grammar.create (Lexer.make ())
let expr_eoi = Grammar.Entry.create gram "expr_eoi"

EXTEND
  GLOBAL: expr_eoi;
  expr_eoi:
    [ [ e = LIST0 expr -> make_seq e ] ]
  ;
  expr:
    [ [ "<"; t = tag; ">"; e = LIST0 expr; "</"; t' = tag; ">" ->
        if t = t' then give_tag (make_seq e) t
        else failwith "Tag not properly nested" 
      | t = LIDENT -> Text t 
      | " " -> Space SPC 
      | "\t" -> Space TAB 
      | "\010" -> Space LF
      | "\013" -> Space CR ] ]
  ;
  tag:
    [ [ LIDENT "B" -> Const "B"
      | LIDENT "EM" -> Const "EM"
      | LIDENT "I" -> Const "I"
      | LIDENT "PL" -> Const "PL"
      | LIDENT "S" -> Const "St"
      | LIDENT "TT" -> Const "TT"
      | LIDENT "U" -> Const "U"
      | LIDENT "r" -> TColor R
      | LIDENT "g" -> TColor G
      | LIDENT "b" -> TColor Bc
      | LIDENT "c" -> TColor C
      | LIDENT "m" -> TColor M
      | LIDENT "y" -> TColor Y
      | LIDENT "k" -> TColor K
      | LIDENT "w" -> TColor W
      | i = LIDENT -> 
        let ni = int_of_string i in
        if (ni>=0) && (ni<=9) then TSize ni
        else failwith "Size between 0 and 9 expected" ] ]
  ;
END;;
